﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Data;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public abstract class RegressionBenchmark : Benchmark, IRegressionBenchmarkProblemDataGenerator {

    #region properties
    protected string targetVariable;
    protected List<string> inputVariables;
    protected IntRange trainingPartition;
    protected IntRange testPartition;

    public string TargetVariable {
      get { return targetVariable; }
    }

    public List<string> InputVariable {
      get { return inputVariables; }
    }

    public IntRange TrainingPartition {
      get { return trainingPartition; }
    }

    public IntRange TestPartition {
      get { return testPartition; }
    }
    #endregion

    protected static FastRandom rand = new FastRandom();

    protected RegressionBenchmark() { }
    protected RegressionBenchmark(RegressionBenchmark original, Cloner cloner)
      : base(original, cloner) {
    }

    public abstract IDataAnalysisProblemData GenerateProblemData();

    public static List<double> GenerateSteps(DoubleRange range, double stepWidth) {
      return Enumerable.Range(0, (int)Math.Round(((range.End - range.Start) / stepWidth) + 1))
                                      .Select(i => (range.Start + i * stepWidth))
                                      .ToList<double>();
    }

    public static List<double> GenerateUniformDistributedValues(int amount, DoubleRange range) {
      List<double> values = new List<double>();
      for (int i = 0; i < amount; i++) {
        values.Add(rand.NextDouble() * (range.End - range.Start) + range.Start);
      }
      return values;
    }

    public static List<double> GenerateNormalDistributedValues(int amount, double mu, double sigma) {
      List<double> values = new List<double>();
      for (int i = 0; i < amount; i++) {
        values.Add(NormalDistributedRandom.NextDouble(rand, mu, sigma));
      }
      return values;
    }

    public static List<List<double>> GenerateAllCombinationsOfValuesInLists(List<List<double>> sets) {

      var combinations = new List<List<double>>();

      foreach (var value in sets[0])
        combinations.Add(new List<double> { value });

      foreach (var set in sets.Skip(1))
        combinations = AddListToCombinations(combinations, set);

      combinations = (from i in Enumerable.Range(0, sets.Count)
                      select (from list in combinations
                              select list.ElementAt(i)).ToList<double>()).ToList<List<double>>();

      return combinations;
    }

    private static List<List<double>> AddListToCombinations
         (List<List<double>> combinations, List<double> set) {
      var newCombinations = from value in set
                            from combination in combinations
                            select new List<double>(combination) { value };

      return newCombinations.ToList();
    }
  }
}
